@@ -87,8 +87,10 @@ int lnf_mem_init(lnf_mem_t **lnf_memp) {
 	//lnf_mem->hash_ptr = NULL;
 	lnf_mem->sorted = 0;
 	lnf_mem->statistics_mode = 0;
+	lnf_mem->list_mode = 0;
 	lnf_mem->numthreads = 0;
 	lnf_mem->read_cursor = NULL;
+	lnf_mem->hash_table_buckets = HASH_TABLE_INIT_SIZE;
 #ifdef LNF_THREADS
 	if (pthread_mutex_init(&lnf_mem->thread_mutex, NULL) != 0) {
 		free(lnf_mem);
@@ -151,7 +153,7 @@ int lnf_mem_thread_init(lnf_mem_t *lnf_mem) {
 		aggr_callback = &lnf_mem_aggr_callback;
 	}
 
-	if (hash_table_init(&lnf_mem->hash_table[*id], HASH_TABLE_INIT_SIZE,
+	if (hash_table_init(&lnf_mem->hash_table[*id], lnf_mem->hash_table_buckets, 
 			aggr_callback, &lnf_mem_sort_callback, lnf_mem) == NULL) {
 
 		return LNF_ERR_NOMEM;
@@ -163,6 +165,30 @@ int lnf_mem_thread_init(lnf_mem_t *lnf_mem) {
 	return LNF_OK;
 }
 
+/* set options for lnf_mem_t */
+int lnf_mem_setopt(lnf_mem_t *lnf_mem, int opt, void *data, size_t size) {
+
+	switch (opt) {
+		case LNF_OPT_HASHBUCKETS: 
+			if (size != sizeof(int)) {
+				return LNF_ERR_OTHER;
+			}
+			
+			lnf_mem->hash_table_buckets = *((int *)data);
+			break;	
+
+		case LNF_OPT_LISTMODE:
+			lnf_mem->list_mode = 1;
+			break;
+
+		default: 
+			return LNF_ERR_OTHER;
+			break;
+	}
+
+	return LNF_OK;
+}
+
 /*! 
 * add item to linked list 
 * if the node with same ID exists in list then only update 
@@ -560,8 +586,16 @@ int lnf_mem_write_raw(lnf_mem_t *lnf_mem, char *buff, int buffsize) {
 		lnf_mem->thread_status[*id] = LNF_TH_WRITE;
 	}
 
+	if (lnf_mem->list_mode) {
+		if (hash_table_insert_list(&lnf_mem->hash_table[*id], buff, buff + lnf_mem->key_len) == NULL) {
+			return LNF_ERR_NOMEM;
+		} else {
+			return LNF_OK;
+		}
+	}
+
 	/* insert record */
-	if (hash_table_insert(&lnf_mem->hash_table[*id], buff, buff + lnf_mem->key_len) == NULL) {
+	if (hash_table_insert_hash(&lnf_mem->hash_table[*id], buff, buff + lnf_mem->key_len) == NULL) {
 		return LNF_ERR_NOMEM;
 	}
 
@@ -627,8 +661,16 @@ int lnf_mem_write(lnf_mem_t *lnf_mem, lnf_rec_t *rec) {
 	}
 
 
+	if (lnf_mem->list_mode) {
+		if (hash_table_insert_list(&lnf_mem->hash_table[*id], keybuf, valbuf) == NULL) {
+			return LNF_ERR_NOMEM;
+		} else {
+			return LNF_OK;
+		}
+	}
+
 	/* insert record */
-	if (hash_table_insert(&lnf_mem->hash_table[*id], keybuf, valbuf) == NULL) {
+	if (hash_table_insert_hash(&lnf_mem->hash_table[*id], keybuf, valbuf) == NULL) {
 		return LNF_ERR_NOMEM;
 	}
 
@@ -640,9 +682,9 @@ int lnf_mem_write(lnf_mem_t *lnf_mem, lnf_rec_t *rec) {
 		lnf_mem_fill_buf(lnf_mem->key_list, rec, keybuf, pairset);
 
 		/* insert record */
-		if (hash_table_insert(&lnf_mem->hash_table[*id], keybuf, valbuf) == NULL) {
+		if (hash_table_insert_hash(&lnf_mem->hash_table[*id], keybuf, valbuf) == NULL) {
 			return LNF_ERR_NOMEM;
-		}
+		} 
 	}
 
 	return LNF_OK;
@@ -654,7 +696,7 @@ int lnf_mem_merge_threads(lnf_mem_t *lnf_mem) {
 
 	int *id;
 	int id2 = 0;
-	int i, finish, merge;
+	int i, merge;
 
 	id = pthread_getspecific(lnf_mem->thread_id_key);
 
@@ -662,60 +704,62 @@ int lnf_mem_merge_threads(lnf_mem_t *lnf_mem) {
 		return LNF_OK;
 	}
 
-//	printf("READY FOR MERGE %d\n", *id);
+//	printf("\nREADY FOR MERGE * %d\n", *id);
 
 	/* set status of the current thread to "ready for merge" */
 	pthread_mutex_lock(&lnf_mem->thread_mutex);
 	lnf_mem->thread_status[*id] = LNF_TH_MERGE;
 	pthread_mutex_unlock(&lnf_mem->thread_mutex);
 
+	/* thread id 0 have nothing to do */
+	if (*id == 0) {
+		return LNF_OK;
+	}
+
 	for (;;) {
 
-		/* choose somebody who can merge with me */
-		pthread_mutex_lock(&lnf_mem->thread_mutex);
+		/* choose somebody who can I merge to */
+//		printf("WAIT READY FOR MERGE * %d\n", *id);
 
-		finish = 1;
 		merge = 0;
 
-		for (i = *id + 1; i < lnf_mem->numthreads; i++) {
+		pthread_mutex_lock(&lnf_mem->thread_mutex);
+		for (i = 0; i < *id; i++) {
 			if (lnf_mem->thread_status[*id] == LNF_TH_MERGE && lnf_mem->thread_status[i] == LNF_TH_MERGE) {
 				id2 = i; 
-				lnf_mem->thread_status[i] = LNF_TH_MERGED;
-				lnf_mem->thread_status[*id] = LNF_TH_MERGING;
-				finish = 0;
+				lnf_mem->thread_status[*id] = LNF_TH_MERGED;
+				lnf_mem->thread_status[i] = LNF_TH_MERGING;
 				merge = 1;
-			}
-			if (lnf_mem->thread_status[i] != LNF_TH_CLEARED) {
-				finish = 0;
+				break;
 			}
 		}
-
-		/* nothing to merge */
-		if (finish) {
-			pthread_mutex_unlock(&lnf_mem->thread_mutex);
-//			printf("FINISH %d \n", *id );
-			return LNF_OK;
-		}
-
-
 		pthread_mutex_unlock(&lnf_mem->thread_mutex);
 
 		if (merge) {
-//			printf("MERGE %d <- %d [%d] \n", *id, id2, lnf_mem->numthreads);
-
-			if (hash_table_merge(&lnf_mem->hash_table[*id], &lnf_mem->hash_table[id2]) == NULL) {
-//				printf("MERGE FAIL: %d\n", *id);
-				return LNF_ERR_NOMEM;
+//			printf("MERGE %d <- %d [%d] \n", id2, *id, lnf_mem->numthreads);
+		
+			/* we either merge hash tables or join them */
+			if (lnf_mem->list_mode) {
+				if (hash_table_join(&lnf_mem->hash_table[id2], &lnf_mem->hash_table[*id]) == NULL) {
+//					printf("JOIN FAIL: %d\n", *id);
+					return LNF_ERR_NOMEM;
+				}
+			} else {
+				if (hash_table_merge(&lnf_mem->hash_table[id2], &lnf_mem->hash_table[*id]) == NULL) {
+//					printf("MERGE FAIL: %d\n", *id);
+					return LNF_ERR_NOMEM;
+				}
 			}
-//			printf("MERGE %d <- %d [%d] DONE \n", *id, id2, lnf_mem->numthreads);
-//			printf("FREE %d\n", id2);
-			hash_table_free(&lnf_mem->hash_table[id2]);
+//			printf("MERGE %d <- %d [%d] DONE \n", id2, *id, lnf_mem->numthreads);
+			hash_table_free(&lnf_mem->hash_table[*id]);
 
 			pthread_mutex_lock(&lnf_mem->thread_mutex);
-			lnf_mem->thread_status[*id] = LNF_TH_MERGE; 
-			lnf_mem->thread_status[id2] = LNF_TH_CLEARED; 
+			lnf_mem->thread_status[*id] = LNF_TH_CLEARED; 
+			lnf_mem->thread_status[id2] = LNF_TH_MERGE; 
 			pthread_mutex_unlock(&lnf_mem->thread_mutex);
+			return LNF_OK;
 		}
+
 	}
 
 #else 
@@ -763,6 +807,55 @@ int lnf_mem_next_c(lnf_mem_t *lnf_mem, lnf_mem_cursor_t **cursor) {
 	return LNF_OK;
 }
 
+/* set cursot to record identified by key */
+int lnf_mem_lookup_c(lnf_mem_t *lnf_mem, lnf_rec_t *rec, lnf_mem_cursor_t **cursor) {
+
+	char *pval;
+	unsigned long hash;
+	char keybuf[LNF_MAX_KEY_LEN]; 
+
+	if (lnf_mem->thread_status[0] == LNF_TH_EMPTY) {
+		*cursor = NULL;
+		return LNF_EOF;
+	}
+
+	/* build key */
+	lnf_mem_fill_buf(lnf_mem->key_list, rec, keybuf, 0);
+
+	*cursor = (lnf_mem_cursor_t *)hash_table_lookup(&lnf_mem->hash_table[0], keybuf, &pval, &hash);
+
+	if (*cursor == NULL) {
+		return LNF_EOF;
+	}
+
+	return LNF_OK;
+}
+
+/* store in raw format in the memory heap */
+int lnf_mem_lookup_raw_c(lnf_mem_t *lnf_mem, char *buff, int buffsize, lnf_mem_cursor_t **cursor) {
+
+	char *pval;
+	unsigned long hash;
+
+	/* compare the size of buffer - have to macht size of key + val fields */
+	if (buffsize != lnf_mem->key_len + lnf_mem->val_len) {
+		return LNF_ERR_OTHER;
+	}
+
+	if (lnf_mem->thread_status[0] == LNF_TH_EMPTY) {
+		*cursor = NULL;
+		return LNF_EOF;
+	}
+
+	*cursor = (lnf_mem_cursor_t *)hash_table_lookup(&lnf_mem->hash_table[0], buff, &pval, &hash);
+
+	if (*cursor == NULL) {
+		return LNF_EOF;
+	}
+
+	return LNF_OK;
+}
+
 /* read next record from memory heap */
 int lnf_mem_read_c(lnf_mem_t *lnf_mem, lnf_mem_cursor_t *cursor, lnf_rec_t *rec) {
 
